@@ -161,10 +161,8 @@ keyboard* kb_new(Window win, Display *display, int kb_x, int kb_y,
     {
       if (font_is_xft)
       {
-	 DBG("loading xft font from cmd line\n");
 	 kb->render_type = xft;
       }
-      DBG("loading font\n");
       _kb_load_font(kb, font_name );
       font_loaded = 1;
     } 
@@ -315,7 +313,6 @@ keyboard* kb_new(Window win, Display *display, int kb_x, int kb_y,
 	    
 	    switch (context) {
 	      case none:
-		DBG( "none\n");
 		break;
 	      case kbddef: /* 
 		if (strcmp(tmpstr_A, "render") == 0)
@@ -429,7 +426,6 @@ keyboard* kb_new(Window win, Display *display, int kb_x, int kb_y,
 
 		break;
 	      case rowdef: /* no rowdefs as yet */
-		DBG( "row\n");
 		break;
 	      case keydef:
 		if (strcmp(tmpstr_A, "default") == 0)
@@ -475,6 +471,17 @@ keyboard* kb_new(Window win, Display *display, int kb_x, int kb_y,
 
 		else if (strcmp(tmpstr_A, "height") == 0)
 		  tmp_but->c_height = atoi(tmpstr_C);
+                else if (strcmp(tmpstr_A, "obey_capslock") == 0)
+		{
+		  if (strcmp(tmpstr_C, "yes") == 0)
+		    tmp_but->options |= OPT_OBEYCAPS;
+		  else if (strcmp(tmpstr_C, "no") == 0)
+		    tmp_but->options &= ~OPT_OBEYCAPS;
+		  else
+		  {
+		    perror("invalid value for obey_capslock\n"); exit(1);
+		  }
+		}
 		break;
 	    }
 	    
@@ -663,14 +670,11 @@ void kb_size(keyboard *kb)
 	  tmp_but = (button *)ip->data;
 
 	  tmp_but->x = cx; /*remember relative to holding box ! */
-	  DBG("1> button is %i, %i, %i\n", tmp_but->x, tmp_but->y, tmp_but->act_width);
 
 	  but_total_width = tmp_but->c_width+(2*tmp_but->b_size);
 
 	  tmp_but->x_pad = (int)(((float)but_total_width/tmp_box->min_width)
 	    * kb->vbox->act_width);
-	  DBG("%i, %i, %i\n", but_total_width, tmp_box->min_width,
-		 kb->vbox->act_width); /* tmp_box is broke */
 
 	  tmp_but->x_pad -= but_total_width;
 
@@ -687,7 +691,6 @@ void kb_size(keyboard *kb)
 	  /*  hack for using all screen space */
 	  if (listp->next == NULL) tmp_but->act_height--; 
 
-	  DBG("2> button is %i, %i\n", tmp_but->x, tmp_but->y);
 	}
       
       /*  another hack for using up all space */
@@ -956,15 +959,14 @@ void kb_send_keypress(button *b)
   {
     if (b->kb->state & KB_STATE_SHIFT && b->kb->state & KB_STATE_CAPS)
       {
-	if (b->default_ks >= 0x061 && b->default_ks <= 0x07a)
+	if (b->options & OPT_OBEYCAPS)
 	  ks = b->default_ks;
 	else 
 	  ks = b->shift_ks;
       }
     else if (b->kb->state & KB_STATE_CAPS)
       {
-	/* Keycodes for XK_a to XK_z, need to add il8n chars ! */
-	if (b->default_ks >= 0x061 && b->default_ks <= 0x07a)
+	if (b->options & OPT_OBEYCAPS)
 	  ks = b->shift_ks;
       } else ks = b->shift_ks;
   }
