@@ -38,16 +38,68 @@
 #define WIN_OVERIDE_AWAYS_TOP 0
 #define WIN_OVERIDE_NOT_AWAYS_TOP 1
 
-/*
-
-TODO:
-
-- internalisation problems with ALT etc *fixed?*
-- show slided keys ???
-*/
-
 Display* display; /* ack globals due to sighandlers - another way ? */	
-Window win;
+Window   win;
+int      screen_num;	
+
+enum {
+  WM_UNKNOWN,
+  WM_EHWM_UNKNOWN,
+  WM_METACITY,
+  WM_MATCHBOX,
+};
+
+static unsigned char*
+get_current_window_manager_name (void)
+{
+  Atom utf8_string, atom, atom_check, type;
+  int result;
+  unsigned char *retval;
+  int format;
+  long nitems;
+  long bytes_after;
+  unsigned char *val;
+  Window *xwindow = NULL;
+
+  atom_check = XInternAtom (display, "_NET_SUPPORTING_WM_CHECK", False);
+
+  XGetWindowProperty (display, 
+		      RootWindow(display, DefaultScreen(display)),
+		      atom_check,
+		      0, 16L, False, XA_WINDOW, &type, &format,
+		      &nitems, &bytes_after, (unsigned char **)&xwindow);
+
+  if (xwindow == NULL)
+      return NULL;
+
+
+  utf8_string = XInternAtom (display, "UTF8_STRING", False);
+  atom = XInternAtom (display, "_NET_WM_NAME", False);
+
+  result = XGetWindowProperty (display,
+		  	       *xwindow,
+			       atom,
+			       0, 1000L,
+			       False, utf8_string,
+			       &type, &format, &nitems,
+			       &bytes_after, (unsigned char **)&val);
+
+  if (result != Success)
+    return NULL;
+
+  if (type != utf8_string || format !=8 || nitems == 0)
+    {
+      if (val) XFree (val);
+      return NULL;
+    }
+
+  retval = strdup (val);
+
+  XFree (val);
+
+  return retval;
+}
+
 
 void handle_sig(int sig)
 {
@@ -96,12 +148,25 @@ void usage(void)
 
 int main(int argc, char **argv)
 {
-   int screen_num;	
-
    char *window_name = "xkbd";
 
    char *icon_name = "xkbd";
 
+#define PROP_MOTIF_WM_HINTS_ELEMENTS    5
+#define MWM_HINTS_DECORATIONS          (1L << 1)
+#define MWM_DECOR_BORDER               (1L << 1)
+
+   typedef struct
+   {
+     unsigned long       flags;
+     unsigned long       functions;
+     unsigned long       decorations;
+     long                inputMode;
+     unsigned long       status;
+   } PropMotifWmHints ;
+   
+   PropMotifWmHints *mwm_hints;
+
    XSizeHints size_hints;
    XWMHints *wm_hints;
 
@@ -109,6 +174,9 @@ int main(int argc, char **argv)
    
    Xkbd *kb = NULL;
 
+   char *wm_name;
+   int wm_type = WM_UNKNOWN;
+
    char *geometry = NULL;
    int xret=0, yret=0, wret=0, hret=0;
    char *conf_file = NULL;
@@ -174,24 +242,29 @@ int main(int argc, char **argv)
    }
 
    display = XOpenDisplay(display_name);
-   screen_num = DefaultScreen(display);
 
    if (display != NULL) 
    {
-
       Atom wm_protocols[]={ 
 	 XInternAtom(display, "WM_DELETE_WINDOW",False),
 	 XInternAtom(display, "WM_PROTOCOLS",False),
 	 XInternAtom(display, "WM_NORMAL_HINTS", False),
       };
 
-
       Atom window_type_atom =
 	 XInternAtom(display, "_NET_WM_WINDOW_TYPE" , False);
       Atom window_type_toolbar_atom =
 	 XInternAtom(display, "_NET_WM_WINDOW_TYPE_TOOLBAR",False);
+      Atom mwm_atom =
+	XInternAtom(display, "_MOTIF_WM_HINTS",False);
 
-	/* HACK to get libvirtkeys to work without mode_switch */
+      Atom window_type_dock_atom = 
+	XInternAtom(display, "_NET_WM_WINDOW_TYPE_DOCK",False);
+
+
+      /* HACK to get libvirtkeys to work without mode_switch */
+
+      screen_num = DefaultScreen(display);
 
       if  (XKeysymToKeycode(display, XK_Mode_switch) == 0)
 	{
@@ -207,9 +280,25 @@ int main(int argc, char **argv)
 		XChangeKeyboardMapping(display, 
 				       keycode, 1,
 				       &mode_switch_ksym, 1);
+		XSync(display, False);
 	      }
       }
-      XSync(display, False);
+      
+      wm_name = get_current_window_manager_name ();
+      use_normal_win = True;
+      
+      if (wm_name)
+	{
+	  wm_type = WM_EHWM_UNKNOWN;
+	  if (!strcmp(wm_name, "metacity"))
+	    wm_type = WM_METACITY;
+	  else if (!strcmp(wm_name, "matchbox"))
+	    {
+
+	      use_normal_win = False;
+	      wm_type = WM_MATCHBOX;
+	    }
+	}
        
       win = XCreateSimpleWindow(display,
 				RootWindow(display, screen_num),
@@ -217,12 +306,22 @@ int main(int argc, char **argv)
 				300, 300,
 				0, BlackPixel(display, screen_num),
 				WhitePixel(display, screen_num));
-       
+      
       if (geometry != NULL)
-      {
-	 XParseGeometry(geometry, &xret, &yret, &wret, &hret );
-      }
-
+	{
+	  XParseGeometry(geometry, &xret, &yret, &wret, &hret );
+	}
+      else
+	{
+	  if (wm_type != WM_MATCHBOX)
+	    {
+	      wret = DisplayWidth(display, screen_num);
+	      hret = DisplayHeight(display, screen_num)/4;
+	      xret = 0;
+	      yret = DisplayHeight(display, screen_num) - hret;
+	    }
+	}
+      
       /* check for user selected keyboard conf file */
 
       if (conf_file == NULL)
@@ -270,7 +369,23 @@ int main(int argc, char **argv)
       wm_hints->input = False;
       wm_hints->flags = InputHint;
       XSetWMHints(display, win, wm_hints );
+
+      /* Tell the WM we dont want no borders */
+      mwm_hints = malloc(sizeof(PropMotifWmHints));
+      memset(mwm_hints, 0, sizeof(PropMotifWmHints));
+      
+      mwm_hints->flags = MWM_HINTS_DECORATIONS;
+      mwm_hints->decorations = 0;
+
+
+      XChangeProperty(display, win, mwm_atom, 
+		      XA_ATOM, 32, PropModeReplace, 
+		      (unsigned char *)mwm_hints, 
+		      PROP_MOTIF_WM_HINTS_ELEMENTS);
+      
+      free(mwm_hints);
     
+
       XSetWMProtocols(display, win, wm_protocols, sizeof(wm_protocols) / 
 		      sizeof(Atom));
 
